@@ -1,7 +1,8 @@
 /* io.c: i/o routines for the ed line editor */
 /*  GNU ed - The GNU line editor.
     Copyright (C) 1993, 1994 Andrew Moore, Talke Studio
-    Copyright (C) 2006, 2007, 2008, 2009 Antonio Diaz Diaz.
+    Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012
+    Free Software Foundation, Inc.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,7 +26,7 @@
 
 
 /* print text to stdout */
-static char put_tty_line( const char *s, int len, const int gflags )
+static void put_tty_line( const char * p, int len, const int gflags )
   {
   const char escapes[] = "\a\b\f\n\r\t\v\\";
   const char escchars[] = "abfnrtv\\";
@@ -34,7 +35,7 @@ static char put_tty_line( const char *s, int len, const int gflags )
   if( gflags & GNP ) { printf( "%d\t", current_addr() ); col = 8; }
   while( --len >= 0 )
     {
-    const unsigned char ch = *s++;
+    const unsigned char ch = *p++;
     if( !( gflags & GLS ) ) putchar( ch );
     else
       {
@@ -42,9 +43,9 @@ static char put_tty_line( const char *s, int len, const int gflags )
       if( ch >= 32 && ch <= 126 && ch != '\\' ) putchar( ch );
       else
         {
-        char *cp = strchr( escapes, ch );
+        char * const p = strchr( escapes, ch );
         ++col; putchar('\\');
-        if( cp ) putchar( escchars[cp-escapes] );
+        if( ch && p ) putchar( escchars[p-escapes] );
         else
           {
           col += 2;
@@ -57,80 +58,85 @@ static char put_tty_line( const char *s, int len, const int gflags )
     }
   if( !traditional() && ( gflags & GLS ) ) putchar('$');
   putchar('\n');
-  return 1;
   }
 
 
 /* print a range of lines to stdout */
-char display_lines( int from, const int to, const int gflags )
+bool display_lines( int from, const int to, const int gflags )
   {
-  line_t *ep = search_line_node( inc_addr( to ) );
-  line_t *bp = search_line_node( from );
+  line_t * const ep = search_line_node( inc_addr( to ) );
+  line_t * bp = search_line_node( from );
 
-  if( !from ) { set_error_msg( "Invalid address" ); return 0; }
+  if( !from ) { set_error_msg( "Invalid address" ); return false; }
   while( bp != ep )
     {
-    char *s = get_sbuf_line( bp );
-    if( !s ) return 0;
+    const char * const s = get_sbuf_line( bp );
+    if( !s ) return false;
     set_current_addr( from++ );
-    if( !put_tty_line( s, bp->len, gflags ) ) return 0;
+    put_tty_line( s, bp->len, gflags );
     bp = bp->q_forw;
     }
-  return 1;
+  return true;
   }
 
 
 /* return the parity of escapes at the end of a string */
-static char trailing_escape( const char * const s, int len )
+static bool trailing_escape( const char * const s, int len )
   {
-  char parity = 0;
-  while( --len >= 0 && s[len] == '\\' ) parity = !parity;
-  return parity;
+  bool odd_escape = false;
+  while( --len >= 0 && s[len] == '\\' ) odd_escape = !odd_escape;
+  return odd_escape;
   }
 
 
-/* get an extended line from stdin */
-const char *get_extended_line( const char *ibufp, int *lenp, const char nonl )
+/* If *ibufpp contains an escaped newline, get an extended line (one
+   with escaped newlines) from stdin */
+bool get_extended_line( const char ** const ibufpp, int * const lenp,
+                        const bool strip_escaped_newlines )
   {
-  static char *buf = 0;
+  static char * buf = 0;
   static int bufsz = 0;
   int len;
 
-  for( len = 0; ibufp[len++] != '\n'; ) ;
-  if( len < 2 || !trailing_escape( ibufp, len - 1 ) )
-    { if( lenp ) *lenp = len; return ibufp; }
-  if( !resize_buffer( &buf, &bufsz, len ) ) return 0;
-  memcpy( buf, ibufp, len );
-  --len; buf[len-1] = '\n';		/* strip trailing esc */
-  if( nonl ) --len;			/* strip newline */
-  while( 1 )
+  for( len = 0; (*ibufpp)[len++] != '\n'; ) ;
+  if( len < 2 || !trailing_escape( *ibufpp, len - 1 ) )
+    { if( lenp ) *lenp = len; return true; }
+  if( !resize_buffer( &buf, &bufsz, len ) ) return false;
+  memcpy( buf, *ibufpp, len );
+  --len; buf[len-1] = '\n';			/* strip trailing esc */
+  if( strip_escaped_newlines ) --len;		/* strip newline */
+  while( true )
     {
     int len2;
-    if( !( ibufp = get_tty_line( &len2 ) ) ) return 0;
-    if( len2 == 0 || ibufp[len2-1] != '\n' )
-      { set_error_msg( "Unexpected end-of-file" ); return 0; }
-    if( !resize_buffer( &buf, &bufsz, len + len2 ) ) return 0;
-    memcpy( buf + len, ibufp, len2 );
+    const char * const s = get_tty_line( &len2 );
+    if( !s ) return false;
+    if( len2 == 0 || s[len2-1] != '\n' )
+      { set_error_msg( "Unexpected end-of-file" ); return false; }
+    if( !resize_buffer( &buf, &bufsz, len + len2 ) ) return false;
+    memcpy( buf + len, s, len2 );
     len += len2;
     if( len2 < 2 || !trailing_escape( buf, len - 1 ) ) break;
-    --len; buf[len-1] = '\n';		/* strip trailing esc */
-    if( nonl ) --len;			/* strip newline */
+    --len; buf[len-1] = '\n';			/* strip trailing esc */
+    if( strip_escaped_newlines ) --len;		/* strip newline */
     }
-  if( !resize_buffer( &buf, &bufsz, len + 1 ) ) return 0;
+  if( !resize_buffer( &buf, &bufsz, len + 1 ) ) return false;
   buf[len] = 0;
+  *ibufpp = buf;
   if( lenp ) *lenp = len;
-  return buf;
+  return true;
   }
 
 
-/* read a line of text from stdin; return pointer to buffer and line length */
-const char *get_tty_line( int *lenp )
+/* Read a line of text from stdin.
+   Return pointer to buffer and line size (uncluding trailing newline
+   if it exists) */
+const char * get_tty_line( int * const sizep )
   {
-  static char *buf = 0;
+  static char * buf = 0;
   static int bufsz = 0;
   int i = 0, oi = -1;
 
-  while( 1 )
+  while( true )
     {
     const int c = getchar();
     if( c == EOF )
@@ -138,36 +144,39 @@ const char *get_tty_line( int *lenp )
       if( ferror( stdin ) )
         {
         show_strerror( "stdin", errno ); set_error_msg( "Cannot read stdin" );
-        clearerr( stdin ); if( lenp ) *lenp = 0;
+        clearerr( stdin ); if( sizep ) *sizep = 0;
         return 0;
         }
       else
         {
         clearerr( stdin ); if( i != oi ) { oi = i; continue; }
-        if( i ) buf[i] = 0; if( lenp ) *lenp = i;
+        if( i ) buf[i] = 0; if( sizep ) *sizep = i;
         return buf;
         }
       }
     else
       {
       if( !resize_buffer( &buf, &bufsz, i + 2 ) )
-        { if( lenp ) *lenp = 0; return 0; }
+        { if( sizep ) *sizep = 0; return 0; }
       buf[i++] = c; if( !c ) set_binary(); if( c != '\n' ) continue;
-      buf[i] = 0; if( lenp ) *lenp = i;
+      buf[i] = 0; if( sizep ) *sizep = i;
       return buf;
       }
     }
   }
 
 
-/* read a line of text from a stream */
-static const char * read_stream_line( FILE *fp, int *lenp, char *newline_added_now )
+/* Read a line of text from a stream.
+   Return pointer to buffer and line size (uncluding trailing newline
+   if it exists and is not added now) */
+static const char * read_stream_line( FILE * const fp, int * const sizep,
+                                      bool * const newline_added_nowp )
   {
-  static char *buf = 0;
+  static char * buf = 0;
   static int bufsz = 0;
   int c, i = 0;
 
-  while( 1 )
+  while( true )
     {
     if( !resize_buffer( &buf, &bufsz, i + 2 ) ) return 0;
     c = getc( fp ); if( c == EOF ) break;
@@ -185,57 +194,63 @@ static const char * read_stream_line( FILE *fp, int *lenp, char *newline_added_n
       }
     else if( i )
       {
-      buf[i] = '\n'; buf[i+1] = 0; *newline_added_now = 1;
+      buf[i] = '\n'; buf[i+1] = 0; *newline_added_nowp = true;
       if( !isbinary() ) ++i;
       }
     }
-  *lenp = i;
+  *sizep = i;
   return buf;
   }
 
 
-/* read a stream into the editor buffer; return size of data read */
-static long read_stream( FILE *fp, const int addr )
+/* read a stream into the editor buffer; return total size of data read */
+static long read_stream( FILE * const fp, const int addr )
   {
-  line_t *lp = search_line_node( addr );
-  undo_t *up = 0;
-  long size = 0;
-  const char o_isbinary = isbinary();
-  const char appended = ( addr == last_addr() );
-  char newline_added_now = 0;
+  line_t * lp = search_line_node( addr );
+  undo_t * up = 0;
+  long total_size = 0;
+  const bool o_isbinary = isbinary();
+  const bool appended = ( addr == last_addr() );
+  bool newline_added_now = false;
 
   set_current_addr( addr );
-  while( 1 )
+  while( true )
     {
-    int len = 0;
-    const char *buf = read_stream_line( fp, &len, &newline_added_now );
-    if( !buf ) return -1;
-    if( len > 0 ) size += len; else break;
+    int size = 0;
+    const char * const s = read_stream_line( fp, &size, &newline_added_now );
+    if( !s ) return -1;
+    if( size > 0 ) total_size += size;
+    else break;
     disable_interrupts();
-    if( !put_sbuf_line( buf, current_addr() ) )
+    if( !put_sbuf_line( s, size + newline_added_now, current_addr() ) )
       { enable_interrupts(); return -1; }
     lp = lp->q_forw;
     if( up ) up->tail = lp;
-    else if( !( up = push_undo_atom( UADD, -1, -1 ) ) )
-      { enable_interrupts(); return -1; }
+    else
+      {
+      up = push_undo_atom( UADD, current_addr(), current_addr() );
+      if( !up ) { enable_interrupts(); return -1; }
+      }
     enable_interrupts();
     }
-  if( addr && appended && size && o_isbinary && newline_added() )
+  if( addr && appended && total_size && o_isbinary && newline_added() )
     fputs( "Newline inserted\n", stderr );
-  else if( newline_added_now && appended )
+  else if( newline_added_now && ( !appended || !isbinary() ) )
     fputs( "Newline appended\n", stderr );
-  if( isbinary() && !o_isbinary && newline_added_now && !appended ) ++size;
-  if( !size ) newline_added_now = 1;
+  if( isbinary() && !o_isbinary && newline_added_now && !appended )
+    ++total_size;
+  if( !total_size ) newline_added_now = true;
   if( appended && newline_added_now ) set_newline_added();
-  return size;
+  return total_size;
   }
 
 
 /* read a named file/pipe into the buffer; return line count */
-int read_file( const char *filename, const int addr )
+int read_file( const char * const filename, const int addr )
   {
-  FILE *fp;
+  FILE * fp;
   long size;
+  int ret;
 
   if( *filename == '!' ) fp = popen( filename + 1, "r" );
   else fp = fopen( strip_escapes( filename ), "r" );
@@ -245,8 +260,10 @@ int read_file( const char *filename, const int addr )
     set_error_msg( "Cannot open input file" );
     return -1;
     }
-  if( ( size = read_stream( fp, addr ) ) < 0 ) return -1;
-  if( ( (*filename == '!' ) ? pclose( fp ) : fclose( fp ) ) < 0 )
+  size = read_stream( fp, addr );
+  if( size < 0 ) return -1;
+  if( *filename == '!' ) ret = pclose( fp ); else ret = fclose( fp );
+  if( ret != 0 )
     {
     show_strerror( filename, errno );
     set_error_msg( "Cannot close input file" );
@@ -258,22 +275,22 @@ int read_file( const char *filename, const int addr )
 
 
 /* write a range of lines to a stream */
-static long write_stream( FILE *fp, int from, const int to )
+static long write_stream( FILE * const fp, int from, const int to )
   {
-  line_t *lp = search_line_node( from );
+  line_t * lp = search_line_node( from );
   long size = 0;
 
   while( from && from <= to )
     {
     int len;
-    char *s = get_sbuf_line( lp );
-    if( !s ) return -1;
+    char * p = get_sbuf_line( lp );
+    if( !p ) return -1;
     len = lp->len;
     if( from != last_addr() || !isbinary() || !newline_added() )
-      s[len++] = '\n';
+      p[len++] = '\n';
     size += len;
     while( --len >= 0 )
-      if( fputc( *s++, fp ) < 0 )
+      if( fputc( *p++, fp ) == EOF )
         {
         show_strerror( 0, errno );
         set_error_msg( "Cannot write file" );
@@ -289,8 +306,9 @@ static long write_stream( FILE *fp, int from, const int to )
 int write_file( const char * const filename, const char * const mode,
                 const int from, const int to )
   {
-  FILE *fp;
+  FILE * fp;
   long size;
+  int ret;
 
   if( *filename == '!' ) fp = popen( filename + 1, "w" );
   else fp = fopen( strip_escapes( filename ), mode );
@@ -300,8 +318,10 @@ int write_file( const char * const filename, const char * const mode,
     set_error_msg( "Cannot open output file" );
     return -1;
     }
-  if( ( size = write_stream( fp, from, to ) ) < 0 ) return -1;
-  if( ( (*filename == '!' ) ? pclose( fp ) : fclose( fp ) ) < 0 )
+  size = write_stream( fp, from, to );
+  if( size < 0 ) return -1;
+  if( *filename == '!' ) ret = pclose( fp ); else ret = fclose( fp );
+  if( ret != 0 )
     {
     show_strerror( filename, errno );
     set_error_msg( "Cannot close output file" );
